#coding:utf-8
"""Модуль nna содержит определения классов-оберток для объектов, используемых
в скриптах автоматизации ректификационной установки (вариант LITE): датчики,
цифровые модули, силовые модули и т.д.
:author: OldBean
16.10.2018"""
import sys
import smbus # В этом модуле - средства для работы с шиной i2c
import time # В этом модуле - задержки
import Adafruit_BMP.BMP085 as bmp # Обслуживание датчика давления
#-------------------------------------------------------------------------------
class Sensor(object):
  """Класс-обертка Sensor используется для всех датчикив, подключаемых к 
  цифровому модулю: термодатчики DS18B20, релейные датчики, любые аналоговые,
  датчики, снабженные 1-Wire адаптером, реализующим протокол прикладного уровня,
  используемый при работе с датчиками DS18B20."""
  def __init__(self, coef = 0, unit = None):
    """Конструктор класса.
    :param coef: коэффициент для перевода кода АЦП в физическое значение
    :param unit: единица измерения (текстовая строка)"""
    self._coef = coef
    self._unit = unit
    self._code = 0x0000 # Текущее значение кода АЦП датчика
  @property
  def v(self): 
    """ Возвращает значение физической величины, измеренное датчиком."""
    return self._code*self._coef
  @property
  def c(self): 
    """ Возвращает код АЦП датчика"""
    return self._code
  @c.setter
  def c(self, code):
    """ Устанавливает текущее значение кода АЦП датчика
    :param code: код АЦП, получаемый от цифрового модуля"""
    self._code = code
  @property
  def u(self):
    """ Возвращает единицу измерения (строку) физической величины, измеряемой
    датчиком"""
    return self._unit
  def __str__(self):
    """ Возвращает строку, содержащую всю информацию о датчике и текущем
    измеренной значении"""
    if self._unit is None:
      return '0x%04x  %d'%(self._code, self._code*self._coef) 
    return '0x%04x  % 3.3f %s'%(self._code, self._code*self._coef, self._unit)
#-------------------------------------------------------------------------------
class DM(object):
  """Класс-обертка DM для цифрового модуля представляет собой контейнеры, 
  содержащие объекты-датчики, подключенные к цифровому модулю, текущие значения
  (коды АЦП) и общие параметры, такие, как адрес на шине i2c и т.д."""
  def __new__(cls, addr, bus = smbus.SMBus(1)):
    try: # Проверим наличие и работоспособность цифрового модуля
      bus.write_byte(addr, 0x00)
      # Все в порядке - создаем объект - цифровой модуль
      return super(DM, cls).__new__(cls) 
    except: 
      print("Цифровой модуль по адресу %x отсутствует или неисправен"%(addr))
      return None
  def __init__(self, addr, bus = smbus.SMBus(1)):
    """ addr - адрес цифрового модуля на шине i2c
       bus - объект-шина SMBus. По умолчанию - SMBus(1)."""
    self._addr = addr
    self._bus = bus
    self._buf = [0]*16 # Буфер данных, полученных от цифрового модуля
    self._sensors = [None]*8
  def update(self):
    """Чтение всего буфера данных цифрового модуля"""
    tries = 100 # Максимальное количество попыток доступа к данным модуля
    while tries > 0: 
      try: # Пытаемся получить данные...
        # Сбрасываем индекс байтов в буфере цифрового модуля
        self._bus.write_byte(self._addr, 0x00)
        for i in range(16): # Теперь считываем буфер байт за байтом
          self._buf[i] = self._bus.read_byte(self._addr)
        break
      except IOError:   # Устройство не готово - подождем 0.1 сек
        time.sleep(0.1) # и опять попробуем
        tries -= 1
    if tries <= 0: # Ну что-то серьезно не так - аварийное заверщение...
      print 'Цифровой модуль не доступен по i2c. Завершение работы.'
      sys.exit(1)
    for i in range(8): # Актуализируем данные подключенных датчиков
      if not self._sensors[i] is None:
        self._sensors[i].c = self._buf[2*i]*256 + self._buf[2*i + 1]
  def addSensor(self, ch = 0, sensor = None):
    """Добавление датчика к цифровому модулю
    :param ch: номер канала,цифрового модуля к которому подключен датчик
    :param sensor: датчик (объект класса Sensor)"""
    if self._sensors[ch] is None:
      self._sensors[ch] = sensor
    else:
      print('Канал цифрового модуля %d уже занят. Подключение отменено.')
  @property
  def sensors(self): 
    """ Возвращает список подключенных датчиков"""
    return self._sensors

  def __str__(self):
    """Сериализация всех данных, полученных от цифрового модуля"""
    s = 'Цифровой модуль:\n'
    for i in range(8):
      # Формируем слово из двух соседних байтов - это полный код АЦП
#      w = self._buf[2*i]*256 + self._buf[2*i + 1]
      s += '%d: '%(i)
      if self._sensors[i] is None: s += 'Нет датчика\n'
      else: s += str(self._sensors[i]) + '\n'
    return s
#-------------------------------------------------------------------------------
class BMP180(object):
  """Класс-обертка для датчика атмосферного давления и температуры BMP180.
     Давления и температура представлены соответствущими свойствами (P и T)
     экземпляра класса. Чтение датчиков происходит при каждом использовании
     свойств в правой части выражения."""
  sensor = None
  def __new__(cls):
    """ Перед созданием объекта В методе __new__ производится проверка
        наличия и работоспособности датчика BMP180"""
    try:
      cls.sensor = bmp.BMP085()
      cls.sensor.read_pressure()
      cls.sensor.read_temperature()
      return super(BMP180, cls).__new__(cls) 
    except: 
      print "Датчик BMP180 отсутствует или неисправен"
      return None

  def __init__(self):
    self._sensor = BMP180.sensor
  @property
  def p(self):
    """Свойство p - значение атмосферного давления в мм.рт.ст."""
    return self._sensor.read_pressure()/133.322 # Паскали -> в мм.рт.ст.

  @property
  def t(self):
    """Свойство t - значение температуры датчика в градусах Цельсия."""
    return self._sensor.read_temperature()
#-------------------------------------------------------------------------------
#if __name__ == '__main__': # Тестирование классов модуля nna

""" Список изменений и история
15.10.2018 - Написаны и отлажены классы-обертки Sensor и DM.
16.10.2018 - В модуль добавлен класс-обертка BMP180, косметические правки,
             в описание классов добавлены подробные комментарии.
"""
